@@ -1,7 +1,7 @@
 package App::CSelUtils;
 
 our $DATE = '2016-08-27'; # DATE
-our $VERSION = '0.04'; # VERSION
+our $VERSION = '0.05'; # VERSION
 
 use 5.010001;
 use strict;
@@ -193,7 +193,7 @@ sub do_actions_on_nodes {
                         }
                     };
                     if ($@) {
-                        warn $@;
+                        #warn $@;
                         $node_res = undef;
                         last;
                     }
@@ -207,6 +207,60 @@ sub do_actions_on_nodes {
     $res;
 }
 
+$SPEC{ddsel} = {
+    v => 1.1,
+    summary => 'Select Perl data structure elements using CSel (CSS-selector-like) syntax',
+    args => {
+        %foosel_common_args,
+        %foosel_struct_action_args,
+    },
+};
+sub ddsel {
+    my %args = @_;
+
+    my $expr = $args{expr};
+    my $actions = $args{actions};
+
+    # parse first so we can bail early on error without having to read the input
+    require Data::CSel;
+    Data::CSel::parse_csel($expr)
+          or return [400, "Invalid CSel expression '$expr'"];
+
+    my $data;
+    if ($args{file} eq '-') {
+        binmode STDIN, ":utf8";
+        $data = eval join("", <>);
+        die if $@;
+    } else {
+        require File::Slurper;
+        $data = eval File::Slurper::read_text($args{file});
+        die if $@;
+    }
+
+    require Data::CSel::WrapStruct;
+    my $tree = Data::CSel::WrapStruct::wrap_struct($data);
+
+    my @matches = Data::CSel::csel(
+        {class_prefixes=>['Data::CSel::WrapStruct']}, $expr, $tree);
+
+    # skip root node itself
+    require Scalar::Util;
+    @matches = grep {
+        Scalar::Util::refaddr($_) ne Scalar::Util::refaddr($tree) } @matches;
+
+    for my $action (@$actions) {
+        if ($action eq 'print') {
+            $action = 'print_func_or_meth:meth:value.func:Data::Dmp::dmp',
+        }
+    }
+
+    require Data::Dmp;
+    App::CSelUtils::do_actions_on_nodes(
+        nodes   => \@matches,
+        actions => $args{actions},
+    );
+}
+
 1;
 
 # ABSTRACT: Utilities related to Data::CSel
@@ -223,7 +277,7 @@ App::CSelUtils - Utilities related to Data::CSel
 
 =head1 VERSION
 
-This document describes version 0.04 of App::CSelUtils (from Perl distribution App-CSelUtils), released on 2016-08-27.
+This document describes version 0.05 of App::CSelUtils (from Perl distribution App-CSelUtils), released on 2016-08-27.
 
 =head1 DESCRIPTION
 
@@ -231,6 +285,8 @@ This distribution contains the following utilities:
 
 =over
 
+=item * L<ddsel>
+
 =item * L<parse-csel>
 
 =back
@@ -238,6 +294,38 @@ This distribution contains the following utilities:
 =head1 FUNCTIONS
 
 
+=head2 ddsel(%args) -> [status, msg, result, meta]
+
+Select Perl data structure elements using CSel (CSS-selector-like) syntax.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<actions> => I<array[str]> (default: ["print"])
+
+Specify action(s) to perform on matching nodes.
+
+=item * B<expr>* => I<str>
+
+=item * B<file> => I<str> (default: "-")
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 parse_csel(%args) -> [status, msg, result, meta]
 
 Parse CSel expression.
